# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: capiscio/v1/simpleguard.proto
# Protobuf Python Version: 6.33.4
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    33,
    4,
    '',
    'capiscio/v1/simpleguard.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from capiscio_mcp._proto.gen.capiscio.v1 import common_pb2 as capiscio_dot_v1_dot_common__pb2
from capiscio_mcp._proto.gen.capiscio.v1 import trust_pb2 as capiscio_dot_v1_dot_trust__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1d\x63\x61piscio/v1/simpleguard.proto\x12\x0b\x63\x61piscio.v1\x1a\x18\x63\x61piscio/v1/common.proto\x1a\x17\x63\x61piscio/v1/trust.proto\"\xf1\x01\n\x0bSignRequest\x12\x18\n\x07payload\x18\x01 \x01(\x0cR\x07payload\x12\x15\n\x06key_id\x18\x02 \x01(\tR\x05keyId\x12\x34\n\x06\x66ormat\x18\x03 \x01(\x0e\x32\x1c.capiscio.v1.SignatureFormatR\x06\x66ormat\x12?\n\x07headers\x18\x04 \x03(\x0b\x32%.capiscio.v1.SignRequest.HeadersEntryR\x07headers\x1a:\n\x0cHeadersEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"|\n\x0cSignResponse\x12\x1c\n\tsignature\x18\x01 \x01(\x0cR\tsignature\x12)\n\x10signature_string\x18\x02 \x01(\tR\x0fsignatureString\x12#\n\rerror_message\x18\x03 \x01(\tR\x0c\x65rrorMessage\"\x9b\x01\n\rVerifyRequest\x12\x18\n\x07payload\x18\x01 \x01(\x0cR\x07payload\x12\x1c\n\tsignature\x18\x02 \x01(\x0cR\tsignature\x12)\n\x10signature_string\x18\x03 \x01(\tR\x0fsignatureString\x12\'\n\x0f\x65xpected_signer\x18\x04 \x01(\tR\x0e\x65xpectedSigner\"\xc0\x01\n\x0eVerifyResponse\x12\x14\n\x05valid\x18\x01 \x01(\x08R\x05valid\x12\x1d\n\nsigner_did\x18\x02 \x01(\tR\tsignerDid\x12\x15\n\x06key_id\x18\x03 \x01(\tR\x05keyId\x12=\n\nvalidation\x18\x04 \x01(\x0b\x32\x1d.capiscio.v1.ValidationResultR\nvalidation\x12#\n\rerror_message\x18\x05 \x01(\tR\x0c\x65rrorMessage\"\xa8\x02\n\x13SignAttachedRequest\x12\x18\n\x07payload\x18\x01 \x01(\x0cR\x07payload\x12\x15\n\x06key_id\x18\x02 \x01(\tR\x05keyId\x12\x34\n\x06\x66ormat\x18\x03 \x01(\x0e\x32\x1c.capiscio.v1.SignatureFormatR\x06\x66ormat\x12G\n\x07headers\x18\x04 \x03(\x0b\x32-.capiscio.v1.SignAttachedRequest.HeadersEntryR\x07headers\x12%\n\x0e\x64\x65tach_payload\x18\x05 \x01(\x08R\rdetachPayload\x1a:\n\x0cHeadersEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"M\n\x14SignAttachedResponse\x12\x10\n\x03jws\x18\x01 \x01(\tR\x03jws\x12#\n\rerror_message\x18\x02 \x01(\tR\x0c\x65rrorMessage\"}\n\x15VerifyAttachedRequest\x12\x10\n\x03jws\x18\x01 \x01(\tR\x03jws\x12)\n\x10\x64\x65tached_payload\x18\x02 \x01(\x0cR\x0f\x64\x65tachedPayload\x12\'\n\x0f\x65xpected_signer\x18\x03 \x01(\tR\x0e\x65xpectedSigner\"\xe2\x01\n\x16VerifyAttachedResponse\x12\x14\n\x05valid\x18\x01 \x01(\x08R\x05valid\x12\x18\n\x07payload\x18\x02 \x01(\x0cR\x07payload\x12\x1d\n\nsigner_did\x18\x03 \x01(\tR\tsignerDid\x12\x15\n\x06key_id\x18\x04 \x01(\tR\x05keyId\x12=\n\nvalidation\x18\x05 \x01(\x0b\x32\x1d.capiscio.v1.ValidationResultR\nvalidation\x12#\n\rerror_message\x18\x06 \x01(\tR\x0c\x65rrorMessage\"\xf4\x01\n\x16GenerateKeyPairRequest\x12\x37\n\talgorithm\x18\x01 \x01(\x0e\x32\x19.capiscio.v1.KeyAlgorithmR\talgorithm\x12\x15\n\x06key_id\x18\x02 \x01(\tR\x05keyId\x12M\n\x08metadata\x18\x03 \x03(\x0b\x32\x31.capiscio.v1.GenerateKeyPairRequest.MetadataEntryR\x08metadata\x1a;\n\rMetadataEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xb5\x02\n\x17GenerateKeyPairResponse\x12\x15\n\x06key_id\x18\x01 \x01(\tR\x05keyId\x12\x1d\n\npublic_key\x18\x02 \x01(\x0cR\tpublicKey\x12\x1f\n\x0bprivate_key\x18\x03 \x01(\x0cR\nprivateKey\x12$\n\x0epublic_key_pem\x18\x04 \x01(\tR\x0cpublicKeyPem\x12&\n\x0fprivate_key_pem\x18\x05 \x01(\tR\rprivateKeyPem\x12\x37\n\talgorithm\x18\x06 \x01(\x0e\x32\x19.capiscio.v1.KeyAlgorithmR\talgorithm\x12#\n\rerror_message\x18\x07 \x01(\tR\x0c\x65rrorMessage\x12\x17\n\x07\x64id_key\x18\x08 \x01(\tR\x06\x64idKey\"}\n\x0eLoadKeyRequest\x12\x1b\n\tfile_path\x18\x01 \x01(\tR\x08\x66ilePath\x12.\n\x06\x66ormat\x18\x02 \x01(\x0e\x32\x16.capiscio.v1.KeyFormatR\x06\x66ormat\x12\x1e\n\npassphrase\x18\x03 \x01(\tR\npassphrase\"\xae\x01\n\x0fLoadKeyResponse\x12\x15\n\x06key_id\x18\x01 \x01(\tR\x05keyId\x12\x37\n\talgorithm\x18\x02 \x01(\x0e\x32\x19.capiscio.v1.KeyAlgorithmR\talgorithm\x12&\n\x0fhas_private_key\x18\x03 \x01(\x08R\rhasPrivateKey\x12#\n\rerror_message\x18\x04 \x01(\tR\x0c\x65rrorMessage\"\xbf\x01\n\x10\x45xportKeyRequest\x12\x15\n\x06key_id\x18\x01 \x01(\tR\x05keyId\x12\x1b\n\tfile_path\x18\x02 \x01(\tR\x08\x66ilePath\x12.\n\x06\x66ormat\x18\x03 \x01(\x0e\x32\x16.capiscio.v1.KeyFormatR\x06\x66ormat\x12\'\n\x0finclude_private\x18\x04 \x01(\x08R\x0eincludePrivate\x12\x1e\n\npassphrase\x18\x05 \x01(\tR\npassphrase\"U\n\x11\x45xportKeyResponse\x12\x1b\n\tfile_path\x18\x01 \x01(\tR\x08\x66ilePath\x12#\n\rerror_message\x18\x02 \x01(\tR\x0c\x65rrorMessage\"*\n\x11GetKeyInfoRequest\x12\x15\n\x06key_id\x18\x01 \x01(\tR\x05keyId\"\xb5\x03\n\x12GetKeyInfoResponse\x12\x15\n\x06key_id\x18\x01 \x01(\tR\x05keyId\x12\x37\n\talgorithm\x18\x02 \x01(\x0e\x32\x19.capiscio.v1.KeyAlgorithmR\talgorithm\x12&\n\x0fhas_private_key\x18\x03 \x01(\x08R\rhasPrivateKey\x12\x1d\n\npublic_key\x18\x04 \x01(\x0cR\tpublicKey\x12$\n\x0epublic_key_pem\x18\x05 \x01(\tR\x0cpublicKeyPem\x12\x35\n\ncreated_at\x18\x06 \x01(\x0b\x32\x16.capiscio.v1.TimestampR\tcreatedAt\x12I\n\x08metadata\x18\x07 \x03(\x0b\x32-.capiscio.v1.GetKeyInfoResponse.MetadataEntryR\x08metadata\x12#\n\rerror_message\x18\x08 \x01(\tR\x0c\x65rrorMessage\x1a;\n\rMetadataEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01*\x8e\x01\n\x0fSignatureFormat\x12 \n\x1cSIGNATURE_FORMAT_UNSPECIFIED\x10\x00\x12 \n\x1cSIGNATURE_FORMAT_JWS_COMPACT\x10\x01\x12\x1d\n\x19SIGNATURE_FORMAT_JWS_JSON\x10\x02\x12\x18\n\x14SIGNATURE_FORMAT_RAW\x10\x03\x32\x83\x05\n\x12SimpleGuardService\x12;\n\x04Sign\x12\x18.capiscio.v1.SignRequest\x1a\x19.capiscio.v1.SignResponse\x12\x41\n\x06Verify\x12\x1a.capiscio.v1.VerifyRequest\x1a\x1b.capiscio.v1.VerifyResponse\x12S\n\x0cSignAttached\x12 .capiscio.v1.SignAttachedRequest\x1a!.capiscio.v1.SignAttachedResponse\x12Y\n\x0eVerifyAttached\x12\".capiscio.v1.VerifyAttachedRequest\x1a#.capiscio.v1.VerifyAttachedResponse\x12\\\n\x0fGenerateKeyPair\x12#.capiscio.v1.GenerateKeyPairRequest\x1a$.capiscio.v1.GenerateKeyPairResponse\x12\x44\n\x07LoadKey\x12\x1b.capiscio.v1.LoadKeyRequest\x1a\x1c.capiscio.v1.LoadKeyResponse\x12J\n\tExportKey\x12\x1d.capiscio.v1.ExportKeyRequest\x1a\x1e.capiscio.v1.ExportKeyResponse\x12M\n\nGetKeyInfo\x12\x1e.capiscio.v1.GetKeyInfoRequest\x1a\x1f.capiscio.v1.GetKeyInfoResponseB\xb6\x01\n\x0f\x63om.capiscio.v1B\x10SimpleguardProtoP\x01ZDgithub.com/capiscio/capiscio-core/pkg/rpc/gen/capiscio/v1;capisciov1\xa2\x02\x03\x43XX\xaa\x02\x0b\x43\x61piscio.V1\xca\x02\x0b\x43\x61piscio\\V1\xe2\x02\x17\x43\x61piscio\\V1\\GPBMetadata\xea\x02\x0c\x43\x61piscio::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'capiscio.v1.simpleguard_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\017com.capiscio.v1B\020SimpleguardProtoP\001ZDgithub.com/capiscio/capiscio-core/pkg/rpc/gen/capiscio/v1;capisciov1\242\002\003CXX\252\002\013Capiscio.V1\312\002\013Capiscio\\V1\342\002\027Capiscio\\V1\\GPBMetadata\352\002\014Capiscio::V1'
  _globals['_SIGNREQUEST_HEADERSENTRY']._loaded_options = None
  _globals['_SIGNREQUEST_HEADERSENTRY']._serialized_options = b'8\001'
  _globals['_SIGNATTACHEDREQUEST_HEADERSENTRY']._loaded_options = None
  _globals['_SIGNATTACHEDREQUEST_HEADERSENTRY']._serialized_options = b'8\001'
  _globals['_GENERATEKEYPAIRREQUEST_METADATAENTRY']._loaded_options = None
  _globals['_GENERATEKEYPAIRREQUEST_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_GETKEYINFORESPONSE_METADATAENTRY']._loaded_options = None
  _globals['_GETKEYINFORESPONSE_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_SIGNATUREFORMAT']._serialized_start=3183
  _globals['_SIGNATUREFORMAT']._serialized_end=3325
  _globals['_SIGNREQUEST']._serialized_start=98
  _globals['_SIGNREQUEST']._serialized_end=339
  _globals['_SIGNREQUEST_HEADERSENTRY']._serialized_start=281
  _globals['_SIGNREQUEST_HEADERSENTRY']._serialized_end=339
  _globals['_SIGNRESPONSE']._serialized_start=341
  _globals['_SIGNRESPONSE']._serialized_end=465
  _globals['_VERIFYREQUEST']._serialized_start=468
  _globals['_VERIFYREQUEST']._serialized_end=623
  _globals['_VERIFYRESPONSE']._serialized_start=626
  _globals['_VERIFYRESPONSE']._serialized_end=818
  _globals['_SIGNATTACHEDREQUEST']._serialized_start=821
  _globals['_SIGNATTACHEDREQUEST']._serialized_end=1117
  _globals['_SIGNATTACHEDREQUEST_HEADERSENTRY']._serialized_start=281
  _globals['_SIGNATTACHEDREQUEST_HEADERSENTRY']._serialized_end=339
  _globals['_SIGNATTACHEDRESPONSE']._serialized_start=1119
  _globals['_SIGNATTACHEDRESPONSE']._serialized_end=1196
  _globals['_VERIFYATTACHEDREQUEST']._serialized_start=1198
  _globals['_VERIFYATTACHEDREQUEST']._serialized_end=1323
  _globals['_VERIFYATTACHEDRESPONSE']._serialized_start=1326
  _globals['_VERIFYATTACHEDRESPONSE']._serialized_end=1552
  _globals['_GENERATEKEYPAIRREQUEST']._serialized_start=1555
  _globals['_GENERATEKEYPAIRREQUEST']._serialized_end=1799
  _globals['_GENERATEKEYPAIRREQUEST_METADATAENTRY']._serialized_start=1740
  _globals['_GENERATEKEYPAIRREQUEST_METADATAENTRY']._serialized_end=1799
  _globals['_GENERATEKEYPAIRRESPONSE']._serialized_start=1802
  _globals['_GENERATEKEYPAIRRESPONSE']._serialized_end=2111
  _globals['_LOADKEYREQUEST']._serialized_start=2113
  _globals['_LOADKEYREQUEST']._serialized_end=2238
  _globals['_LOADKEYRESPONSE']._serialized_start=2241
  _globals['_LOADKEYRESPONSE']._serialized_end=2415
  _globals['_EXPORTKEYREQUEST']._serialized_start=2418
  _globals['_EXPORTKEYREQUEST']._serialized_end=2609
  _globals['_EXPORTKEYRESPONSE']._serialized_start=2611
  _globals['_EXPORTKEYRESPONSE']._serialized_end=2696
  _globals['_GETKEYINFOREQUEST']._serialized_start=2698
  _globals['_GETKEYINFOREQUEST']._serialized_end=2740
  _globals['_GETKEYINFORESPONSE']._serialized_start=2743
  _globals['_GETKEYINFORESPONSE']._serialized_end=3180
  _globals['_GETKEYINFORESPONSE_METADATAENTRY']._serialized_start=1740
  _globals['_GETKEYINFORESPONSE_METADATAENTRY']._serialized_end=1799
  _globals['_SIMPLEGUARDSERVICE']._serialized_start=3328
  _globals['_SIMPLEGUARDSERVICE']._serialized_end=3971
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from capiscio_mcp._proto.gen.capiscio.v1 import did_pb2 as capiscio_dot_v1_dot_did__pb2


class DIDServiceStub(object):
    """DIDService handles did:web operations
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Parse = channel.unary_unary(
                '/capiscio.v1.DIDService/Parse',
                request_serializer=capiscio_dot_v1_dot_did__pb2.ParseDIDRequest.SerializeToString,
                response_deserializer=capiscio_dot_v1_dot_did__pb2.ParseDIDResponse.FromString,
                _registered_method=True)
        self.NewAgentDID = channel.unary_unary(
                '/capiscio.v1.DIDService/NewAgentDID',
                request_serializer=capiscio_dot_v1_dot_did__pb2.NewAgentDIDRequest.SerializeToString,
                response_deserializer=capiscio_dot_v1_dot_did__pb2.NewAgentDIDResponse.FromString,
                _registered_method=True)
        self.NewCapiscIOAgentDID = channel.unary_unary(
                '/capiscio.v1.DIDService/NewCapiscIOAgentDID',
                request_serializer=capiscio_dot_v1_dot_did__pb2.NewCapiscIOAgentDIDRequest.SerializeToString,
                response_deserializer=capiscio_dot_v1_dot_did__pb2.NewAgentDIDResponse.FromString,
                _registered_method=True)
        self.DocumentURL = channel.unary_unary(
                '/capiscio.v1.DIDService/DocumentURL',
                request_serializer=capiscio_dot_v1_dot_did__pb2.DocumentURLRequest.SerializeToString,
                response_deserializer=capiscio_dot_v1_dot_did__pb2.DocumentURLResponse.FromString,
                _registered_method=True)
        self.IsAgentDID = channel.unary_unary(
                '/capiscio.v1.DIDService/IsAgentDID',
                request_serializer=capiscio_dot_v1_dot_did__pb2.IsAgentDIDRequest.SerializeToString,
                response_deserializer=capiscio_dot_v1_dot_did__pb2.IsAgentDIDResponse.FromString,
                _registered_method=True)


class DIDServiceServicer(object):
    """DIDService handles did:web operations
    """

    def Parse(self, request, context):
        """Parse a did:web identifier
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def NewAgentDID(self, request, context):
        """Construct a new agent DID
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def NewCapiscIOAgentDID(self, request, context):
        """Construct a Capiscio registry DID
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DocumentURL(self, request, context):
        """Get the document URL for a DID
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def IsAgentDID(self, request, context):
        """Check if a DID is an agent DID
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_DIDServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Parse': grpc.unary_unary_rpc_method_handler(
                    servicer.Parse,
                    request_deserializer=capiscio_dot_v1_dot_did__pb2.ParseDIDRequest.FromString,
                    response_serializer=capiscio_dot_v1_dot_did__pb2.ParseDIDResponse.SerializeToString,
            ),
            'NewAgentDID': grpc.unary_unary_rpc_method_handler(
                    servicer.NewAgentDID,
                    request_deserializer=capiscio_dot_v1_dot_did__pb2.NewAgentDIDRequest.FromString,
                    response_serializer=capiscio_dot_v1_dot_did__pb2.NewAgentDIDResponse.SerializeToString,
            ),
            'NewCapiscIOAgentDID': grpc.unary_unary_rpc_method_handler(
                    servicer.NewCapiscIOAgentDID,
                    request_deserializer=capiscio_dot_v1_dot_did__pb2.NewCapiscIOAgentDIDRequest.FromString,
                    response_serializer=capiscio_dot_v1_dot_did__pb2.NewAgentDIDResponse.SerializeToString,
            ),
            'DocumentURL': grpc.unary_unary_rpc_method_handler(
                    servicer.DocumentURL,
                    request_deserializer=capiscio_dot_v1_dot_did__pb2.DocumentURLRequest.FromString,
                    response_serializer=capiscio_dot_v1_dot_did__pb2.DocumentURLResponse.SerializeToString,
            ),
            'IsAgentDID': grpc.unary_unary_rpc_method_handler(
                    servicer.IsAgentDID,
                    request_deserializer=capiscio_dot_v1_dot_did__pb2.IsAgentDIDRequest.FromString,
                    response_serializer=capiscio_dot_v1_dot_did__pb2.IsAgentDIDResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'capiscio.v1.DIDService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('capiscio.v1.DIDService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class DIDService(object):
    """DIDService handles did:web operations
    """

    @staticmethod
    def Parse(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/capiscio.v1.DIDService/Parse',
            capiscio_dot_v1_dot_did__pb2.ParseDIDRequest.SerializeToString,
            capiscio_dot_v1_dot_did__pb2.ParseDIDResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def NewAgentDID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/capiscio.v1.DIDService/NewAgentDID',
            capiscio_dot_v1_dot_did__pb2.NewAgentDIDRequest.SerializeToString,
            capiscio_dot_v1_dot_did__pb2.NewAgentDIDResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def NewCapiscIOAgentDID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/capiscio.v1.DIDService/NewCapiscIOAgentDID',
            capiscio_dot_v1_dot_did__pb2.NewCapiscIOAgentDIDRequest.SerializeToString,
            capiscio_dot_v1_dot_did__pb2.NewAgentDIDResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DocumentURL(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/capiscio.v1.DIDService/DocumentURL',
            capiscio_dot_v1_dot_did__pb2.DocumentURLRequest.SerializeToString,
            capiscio_dot_v1_dot_did__pb2.DocumentURLResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def IsAgentDID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/capiscio.v1.DIDService/IsAgentDID',
            capiscio_dot_v1_dot_did__pb2.IsAgentDIDRequest.SerializeToString,
            capiscio_dot_v1_dot_did__pb2.IsAgentDIDResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



"""Tests for capiscio_mcp._core.lifecycle module."""

import asyncio
import os
import sys
import tempfile
from pathlib import Path
from unittest.mock import AsyncMock, MagicMock, patch
import pytest

from capiscio_mcp._core.lifecycle import (
    get_platform_info,
    get_binary_path,
    get_cache_dir,
    download_binary,
    ensure_binary,
    start_core_process,
    ProcessSupervisor,
    BinaryNotFoundError,
)
from capiscio_mcp._core.version import CORE_MIN_VERSION


class TestGetPlatformInfo:
    """Tests for get_platform_info function."""
    
    def test_returns_tuple(self):
        """Should return (os, arch) tuple."""
        result = get_platform_info()
        assert isinstance(result, tuple)
        assert len(result) == 2
    
    def test_os_is_valid(self):
        """OS should be one of known values."""
        os_name, _ = get_platform_info()
        assert os_name in ("linux", "darwin", "windows")
    
    def test_arch_is_valid(self):
        """Architecture should be one of known values."""
        _, arch = get_platform_info()
        assert arch in ("amd64", "arm64")
    
    @patch("platform.system")
    @patch("platform.machine")
    def test_linux_x86_64(self, mock_machine, mock_system):
        """Linux x86_64 should map correctly."""
        mock_system.return_value = "Linux"
        mock_machine.return_value = "x86_64"
        
        os_name, arch = get_platform_info()
        
        assert os_name == "linux"
        assert arch == "amd64"
    
    @patch("platform.system")
    @patch("platform.machine")
    def test_darwin_arm64(self, mock_machine, mock_system):
        """macOS ARM64 should map correctly."""
        mock_system.return_value = "Darwin"
        mock_machine.return_value = "arm64"
        
        os_name, arch = get_platform_info()
        
        assert os_name == "darwin"
        assert arch == "arm64"
    
    @patch("platform.system")
    @patch("platform.machine")
    def test_windows_amd64(self, mock_machine, mock_system):
        """Windows AMD64 should map correctly."""
        mock_system.return_value = "Windows"
        mock_machine.return_value = "AMD64"
        
        os_name, arch = get_platform_info()
        
        assert os_name == "windows"
        assert arch == "amd64"
    
    @patch("platform.system")
    @patch("platform.machine")
    def test_linux_aarch64(self, mock_machine, mock_system):
        """Linux aarch64 should map to arm64."""
        mock_system.return_value = "Linux"
        mock_machine.return_value = "aarch64"
        
        os_name, arch = get_platform_info()
        
        assert os_name == "linux"
        assert arch == "arm64"


class TestGetCacheDir:
    """Tests for get_cache_dir function."""
    
    def test_returns_path(self):
        """Should return a Path object."""
        result = get_cache_dir()
        assert isinstance(result, Path)
    
    def test_path_exists(self):
        """Cache directory should exist after call."""
        result = get_cache_dir()
        assert result.exists()
    
    def test_path_is_directory(self):
        """Should be a directory."""
        result = get_cache_dir()
        assert result.is_dir()


class TestGetBinaryPath:
    """Tests for get_binary_path function."""
    
    def test_returns_path(self):
        """Should return a Path object."""
        result = get_binary_path()
        assert isinstance(result, Path)
    
    def test_path_contains_capiscio(self):
        """Path should contain 'capiscio'."""
        result = get_binary_path()
        assert "capiscio" in str(result).lower()
    
    def test_binary_name_contains_platform(self):
        """Binary name should contain platform info."""
        result = get_binary_path()
        binary_name = result.name
        os_name, arch_name = get_platform_info()
        
        assert os_name in binary_name
        assert arch_name in binary_name
    
    def test_path_contains_version(self):
        """Path should contain version subdirectory."""
        result = get_binary_path()
        assert CORE_MIN_VERSION in str(result)
    
    def test_custom_version(self):
        """Should use custom version in path."""
        result = get_binary_path(version="2.6.0")
        assert "2.6.0" in str(result)


class TestDownloadBinary:
    """Tests for download_binary function (sync)."""
    
    @patch("requests.get")
    @patch("capiscio_mcp._core.lifecycle.get_binary_path")
    def test_returns_existing_binary(self, mock_path, mock_get):
        """Should return existing binary without download."""
        with tempfile.TemporaryDirectory() as tmpdir:
            binary_path = Path(tmpdir) / "capiscio"
            binary_path.parent.mkdir(parents=True, exist_ok=True)
            binary_path.write_bytes(b"existing binary")
            mock_path.return_value = binary_path
            
            result = download_binary()
            
            assert result == binary_path
            mock_get.assert_not_called()
    
    @patch("requests.get")
    @patch("capiscio_mcp._core.lifecycle.get_binary_path")
    def test_download_success(self, mock_path, mock_get):
        """Successful download should return binary path."""
        with tempfile.TemporaryDirectory() as tmpdir:
            binary_path = Path(tmpdir) / "subdir" / "capiscio"
            mock_path.return_value = binary_path
            
            mock_response = MagicMock()
            mock_response.status_code = 200
            mock_response.iter_content = MagicMock(return_value=[b"binary", b"content"])
            mock_response.raise_for_status = MagicMock()
            mock_get.return_value = mock_response
            
            result = download_binary()
            
            assert result == binary_path
            assert binary_path.exists()
    
    @patch("requests.get")
    @patch("capiscio_mcp._core.lifecycle.get_binary_path")
    def test_download_404_error(self, mock_path, mock_get):
        """404 response should raise error."""
        import requests
        
        with tempfile.TemporaryDirectory() as tmpdir:
            binary_path = Path(tmpdir) / "subdir" / "capiscio"
            mock_path.return_value = binary_path
            
            mock_response = MagicMock()
            mock_response.status_code = 404
            mock_response.raise_for_status.side_effect = requests.exceptions.HTTPError("404")
            mock_get.return_value = mock_response
            
            from capiscio_mcp.errors import CoreConnectionError
            with pytest.raises(CoreConnectionError):
                download_binary()
    
    @patch("requests.get")
    @patch("capiscio_mcp._core.lifecycle.get_binary_path")
    def test_download_sets_executable(self, mock_path, mock_get):
        """Downloaded binary should be executable on Unix."""
        if sys.platform == "win32":
            pytest.skip("Executable test not applicable on Windows")
        
        with tempfile.TemporaryDirectory() as tmpdir:
            binary_path = Path(tmpdir) / "subdir" / "capiscio"
            mock_path.return_value = binary_path
            
            mock_response = MagicMock()
            mock_response.status_code = 200
            mock_response.iter_content = MagicMock(return_value=[b"#!/bin/sh\necho hello"])
            mock_response.raise_for_status = MagicMock()
            mock_get.return_value = mock_response
            
            result = download_binary()
            
            assert os.access(result, os.X_OK)


class TestEnsureBinary:
    """Tests for ensure_binary async function."""
    
    @pytest.mark.asyncio
    async def test_returns_path(self):
        """Should return path to binary."""
        with tempfile.TemporaryDirectory() as tmpdir:
            binary_path = Path(tmpdir) / "capiscio"
            binary_path.write_bytes(b"existing binary")
            
            with patch("capiscio_mcp._core.lifecycle.download_binary", return_value=binary_path):
                result = await ensure_binary()
                assert isinstance(result, Path)
    
    @pytest.mark.asyncio
    async def test_calls_download_binary(self):
        """Should call download_binary in executor when no local override."""
        # Clear any CAPISCIO_BINARY_PATH env var to ensure download path is taken
        with patch.dict(os.environ, {}, clear=True):
            with patch("capiscio_mcp._core.lifecycle.download_binary") as mock_download:
                mock_download.return_value = Path("/tmp/capiscio")
                
                await ensure_binary()
                
                mock_download.assert_called_once()


class TestStartCoreProcess:
    """Tests for start_core_process function."""
    
    @pytest.mark.asyncio
    async def test_returns_process(self):
        """Should return subprocess.Process."""
        with patch("asyncio.create_subprocess_exec") as mock_exec:
            mock_process = MagicMock()
            mock_process.pid = 12345
            mock_exec.return_value = mock_process
            
            result = await start_core_process(Path("/bin/capiscio"), port=50051)
            
            assert result is mock_process
    
    @pytest.mark.asyncio
    async def test_correct_command(self):
        """Should invoke correct command."""
        with patch("asyncio.create_subprocess_exec") as mock_exec:
            mock_process = MagicMock()
            mock_exec.return_value = mock_process
            
            await start_core_process(Path("/bin/capiscio"), port=50051)
            
            call_args = mock_exec.call_args
            cmd = call_args[0]
            
            assert "/bin/capiscio" in cmd[0]
            assert "rpc" in cmd
            assert "--address" in cmd
            assert "localhost:50051" in cmd


class TestProcessSupervisor:
    """Tests for ProcessSupervisor class."""
    
    def test_init(self):
        """Should initialize with correct attributes."""
        supervisor = ProcessSupervisor(
            binary_path=Path("/bin/capiscio"),
            port=50051,
            max_restarts=5,
        )
        
        assert supervisor.binary_path == Path("/bin/capiscio")
        assert supervisor.port == 50051
        assert supervisor.max_restarts == 5
    
    @pytest.mark.asyncio
    async def test_start(self):
        """Should start capiscio-core process."""
        supervisor = ProcessSupervisor(
            binary_path=Path("/bin/capiscio"),
            port=50051,
        )
        
        with patch("capiscio_mcp._core.lifecycle.start_core_process") as mock_start:
            mock_process = MagicMock()
            mock_process.returncode = None
            mock_process.wait = AsyncMock(return_value=0)
            mock_start.return_value = mock_process
            
            await supervisor.start()
            
            mock_start.assert_called_once_with(Path("/bin/capiscio"), 50051)
            assert supervisor._process is mock_process
    
    @pytest.mark.asyncio
    async def test_stop(self):
        """Should stop running process."""
        supervisor = ProcessSupervisor(
            binary_path=Path("/bin/capiscio"),
            port=50051,
        )
        
        mock_process = MagicMock()
        mock_process.returncode = None
        mock_process.terminate = MagicMock()
        mock_process.wait = AsyncMock(return_value=0)
        mock_process.kill = MagicMock()
        
        supervisor._process = mock_process
        supervisor._running = True
        supervisor._supervisor_task = asyncio.create_task(asyncio.sleep(100))
        
        await supervisor.stop()
        
        mock_process.terminate.assert_called_once()
    
    def test_is_running_true(self):
        """is_running should return True for running process."""
        supervisor = ProcessSupervisor(
            binary_path=Path("/bin/capiscio"),
            port=50051,
        )
        
        mock_process = MagicMock()
        mock_process.returncode = None  # Still running
        
        supervisor._process = mock_process
        
        assert supervisor.is_running is True
    
    def test_is_running_false(self):
        """is_running should return False for stopped process."""
        supervisor = ProcessSupervisor(
            binary_path=Path("/bin/capiscio"),
            port=50051,
        )
        
        mock_process = MagicMock()
        mock_process.returncode = 0  # Stopped
        
        supervisor._process = mock_process
        
        assert supervisor.is_running is False
    
    def test_is_running_no_process(self):
        """is_running should return False when no process."""
        supervisor = ProcessSupervisor(
            binary_path=Path("/bin/capiscio"),
            port=50051,
        )
        
        assert supervisor.is_running is False


class TestBinaryNotFoundError:
    """Tests for BinaryNotFoundError exception."""
    
    def test_error_message(self):
        """Error should have descriptive message."""
        error = BinaryNotFoundError("Binary not found for linux/amd64")
        assert "linux" in str(error)
        assert "amd64" in str(error)
    
    def test_is_exception(self):
        """Should be a proper exception."""
        error = BinaryNotFoundError("test")
        assert isinstance(error, Exception)
        
        with pytest.raises(BinaryNotFoundError):
            raise error
    
    def test_inherits_core_connection_error(self):
        """Should inherit from CoreConnectionError."""
        from capiscio_mcp.errors import CoreConnectionError
        error = BinaryNotFoundError("test")
        assert isinstance(error, CoreConnectionError)



"""
Exception types for capiscio-mcp.

Provides typed exceptions for:
- Guard (RFC-006) errors
- Server verification (RFC-007) errors
- Core connection errors
"""

from __future__ import annotations

from typing import Optional, TYPE_CHECKING

if TYPE_CHECKING:
    from capiscio_mcp.types import DenyReason, ServerErrorCode, ServerState


class CapiscioMCPError(Exception):
    """Base exception for all capiscio-mcp errors."""
    pass


# =============================================================================
# Core Connection Errors
# =============================================================================


class CoreConnectionError(CapiscioMCPError):
    """
    Raised when connection to capiscio-core fails.
    
    This includes:
    - Binary download failures
    - Process startup failures
    - gRPC connection failures
    - Health check failures
    """
    pass


class CoreVersionError(CapiscioMCPError):
    """
    Raised when capiscio-core version is incompatible.
    
    capiscio-mcp requires a specific range of capiscio-core versions.
    This error indicates the connected core is outside that range.
    """
    pass


# =============================================================================
# RFC-006: Guard Errors
# =============================================================================


class GuardError(CapiscioMCPError):
    """
    Raised when tool access is denied by the guard.
    
    Per RFC-006, all denials include:
    - reason: Specific denial reason code
    - detail: Human-readable explanation
    - evidence_id: ID of the evidence record for audit
    
    Example:
        try:
            result = await guarded_tool(params)
        except GuardError as e:
            logger.warning(f"Access denied: {e.reason} - {e.detail}")
            logger.info(f"Evidence ID: {e.evidence_id}")
    """
    
    def __init__(
        self,
        reason: "DenyReason",
        detail: str,
        evidence_id: str = "",
        agent_did: Optional[str] = None,
        trust_level: Optional[int] = None,
    ):
        self.reason = reason
        self.detail = detail
        self.evidence_id = evidence_id
        self.agent_did = agent_did
        self.trust_level = trust_level
        
        message = f"{reason.value}: {detail}"
        if evidence_id:
            message += f" (evidence_id={evidence_id})"
        
        super().__init__(message)
    
    def __repr__(self) -> str:
        return (
            f"GuardError(reason={self.reason!r}, detail={self.detail!r}, "
            f"evidence_id={self.evidence_id!r})"
        )


class GuardConfigError(CapiscioMCPError):
    """
    Raised when guard configuration is invalid.
    
    This includes:
    - Invalid trust level values
    - Invalid tool name patterns
    - Conflicting configuration options
    """
    pass


# =============================================================================
# RFC-007: Server Verification Errors
# =============================================================================


class ServerVerifyError(CapiscioMCPError):
    """
    Raised when server identity verification fails.
    
    Per RFC-007, verification can fail for various reasons including:
    - Invalid DID format
    - Badge verification failure
    - Origin/path binding mismatch
    - Trust level insufficient
    
    Example:
        try:
            result = await verify_server(server_did, server_badge)
        except ServerVerifyError as e:
            logger.warning(f"Server verification failed: {e.error_code}")
            if e.state == ServerState.UNVERIFIED_ORIGIN:
                logger.warning("Server did not disclose identity")
    """
    
    def __init__(
        self,
        error_code: "ServerErrorCode",
        detail: str,
        state: Optional["ServerState"] = None,
        server_did: Optional[str] = None,
    ):
        self.error_code = error_code
        self.detail = detail
        self.state = state
        self.server_did = server_did
        
        message = f"{error_code.value}: {detail}"
        super().__init__(message)
    
    def __repr__(self) -> str:
        return (
            f"ServerVerifyError(error_code={self.error_code!r}, "
            f"detail={self.detail!r}, state={self.state!r})"
        )


# =============================================================================
# Evidence Errors
# =============================================================================


class EvidenceError(CapiscioMCPError):
    """
    Raised when evidence logging fails.
    
    Evidence logging should not block tool execution, so this error
    is typically logged rather than raised to callers.
    """
    
    def __init__(self, message: str, evidence_id: Optional[str] = None):
        self.evidence_id = evidence_id
        super().__init__(message)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from capiscio_mcp._proto.gen.capiscio.v1 import revocation_pb2 as capiscio_dot_v1_dot_revocation__pb2


class RevocationServiceStub(object):
    """RevocationService manages revocation lists and checks
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.IsRevoked = channel.unary_unary(
                '/capiscio.v1.RevocationService/IsRevoked',
                request_serializer=capiscio_dot_v1_dot_revocation__pb2.IsRevokedRequest.SerializeToString,
                response_deserializer=capiscio_dot_v1_dot_revocation__pb2.IsRevokedResponse.FromString,
                _registered_method=True)
        self.Revoke = channel.unary_unary(
                '/capiscio.v1.RevocationService/Revoke',
                request_serializer=capiscio_dot_v1_dot_revocation__pb2.RevokeRequest.SerializeToString,
                response_deserializer=capiscio_dot_v1_dot_revocation__pb2.RevokeResponse.FromString,
                _registered_method=True)
        self.Unrevoke = channel.unary_unary(
                '/capiscio.v1.RevocationService/Unrevoke',
                request_serializer=capiscio_dot_v1_dot_revocation__pb2.UnrevokeRequest.SerializeToString,
                response_deserializer=capiscio_dot_v1_dot_revocation__pb2.UnrevokeResponse.FromString,
                _registered_method=True)
        self.ListRevocations = channel.unary_unary(
                '/capiscio.v1.RevocationService/ListRevocations',
                request_serializer=capiscio_dot_v1_dot_revocation__pb2.ListRevocationsRequest.SerializeToString,
                response_deserializer=capiscio_dot_v1_dot_revocation__pb2.ListRevocationsResponse.FromString,
                _registered_method=True)
        self.FetchRevocationList = channel.unary_unary(
                '/capiscio.v1.RevocationService/FetchRevocationList',
                request_serializer=capiscio_dot_v1_dot_revocation__pb2.FetchRevocationListRequest.SerializeToString,
                response_deserializer=capiscio_dot_v1_dot_revocation__pb2.FetchRevocationListResponse.FromString,
                _registered_method=True)
        self.ClearCache = channel.unary_unary(
                '/capiscio.v1.RevocationService/ClearCache',
                request_serializer=capiscio_dot_v1_dot_revocation__pb2.ClearCacheRequest.SerializeToString,
                response_deserializer=capiscio_dot_v1_dot_revocation__pb2.ClearCacheResponse.FromString,
                _registered_method=True)
        self.GetCacheStats = channel.unary_unary(
                '/capiscio.v1.RevocationService/GetCacheStats',
                request_serializer=capiscio_dot_v1_dot_revocation__pb2.GetCacheStatsRequest.SerializeToString,
                response_deserializer=capiscio_dot_v1_dot_revocation__pb2.GetCacheStatsResponse.FromString,
                _registered_method=True)


class RevocationServiceServicer(object):
    """RevocationService manages revocation lists and checks
    """

    def IsRevoked(self, request, context):
        """Check if a key is revoked
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Revoke(self, request, context):
        """Add a revocation entry
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Unrevoke(self, request, context):
        """Remove a revocation entry
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListRevocations(self, request, context):
        """List revoked entries
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FetchRevocationList(self, request, context):
        """Fetch revocation list from URL
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClearCache(self, request, context):
        """Clear the revocation cache
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCacheStats(self, request, context):
        """Get cache statistics
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_RevocationServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'IsRevoked': grpc.unary_unary_rpc_method_handler(
                    servicer.IsRevoked,
                    request_deserializer=capiscio_dot_v1_dot_revocation__pb2.IsRevokedRequest.FromString,
                    response_serializer=capiscio_dot_v1_dot_revocation__pb2.IsRevokedResponse.SerializeToString,
            ),
            'Revoke': grpc.unary_unary_rpc_method_handler(
                    servicer.Revoke,
                    request_deserializer=capiscio_dot_v1_dot_revocation__pb2.RevokeRequest.FromString,
                    response_serializer=capiscio_dot_v1_dot_revocation__pb2.RevokeResponse.SerializeToString,
            ),
            'Unrevoke': grpc.unary_unary_rpc_method_handler(
                    servicer.Unrevoke,
                    request_deserializer=capiscio_dot_v1_dot_revocation__pb2.UnrevokeRequest.FromString,
                    response_serializer=capiscio_dot_v1_dot_revocation__pb2.UnrevokeResponse.SerializeToString,
            ),
            'ListRevocations': grpc.unary_unary_rpc_method_handler(
                    servicer.ListRevocations,
                    request_deserializer=capiscio_dot_v1_dot_revocation__pb2.ListRevocationsRequest.FromString,
                    response_serializer=capiscio_dot_v1_dot_revocation__pb2.ListRevocationsResponse.SerializeToString,
            ),
            'FetchRevocationList': grpc.unary_unary_rpc_method_handler(
                    servicer.FetchRevocationList,
                    request_deserializer=capiscio_dot_v1_dot_revocation__pb2.FetchRevocationListRequest.FromString,
                    response_serializer=capiscio_dot_v1_dot_revocation__pb2.FetchRevocationListResponse.SerializeToString,
            ),
            'ClearCache': grpc.unary_unary_rpc_method_handler(
                    servicer.ClearCache,
                    request_deserializer=capiscio_dot_v1_dot_revocation__pb2.ClearCacheRequest.FromString,
                    response_serializer=capiscio_dot_v1_dot_revocation__pb2.ClearCacheResponse.SerializeToString,
            ),
            'GetCacheStats': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCacheStats,
                    request_deserializer=capiscio_dot_v1_dot_revocation__pb2.GetCacheStatsRequest.FromString,
                    response_serializer=capiscio_dot_v1_dot_revocation__pb2.GetCacheStatsResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'capiscio.v1.RevocationService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('capiscio.v1.RevocationService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class RevocationService(object):
    """RevocationService manages revocation lists and checks
    """

    @staticmethod
    def IsRevoked(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/capiscio.v1.RevocationService/IsRevoked',
            capiscio_dot_v1_dot_revocation__pb2.IsRevokedRequest.SerializeToString,
            capiscio_dot_v1_dot_revocation__pb2.IsRevokedResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Revoke(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/capiscio.v1.RevocationService/Revoke',
            capiscio_dot_v1_dot_revocation__pb2.RevokeRequest.SerializeToString,
            capiscio_dot_v1_dot_revocation__pb2.RevokeResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Unrevoke(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/capiscio.v1.RevocationService/Unrevoke',
            capiscio_dot_v1_dot_revocation__pb2.UnrevokeRequest.SerializeToString,
            capiscio_dot_v1_dot_revocation__pb2.UnrevokeResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListRevocations(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/capiscio.v1.RevocationService/ListRevocations',
            capiscio_dot_v1_dot_revocation__pb2.ListRevocationsRequest.SerializeToString,
            capiscio_dot_v1_dot_revocation__pb2.ListRevocationsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def FetchRevocationList(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/capiscio.v1.RevocationService/FetchRevocationList',
            capiscio_dot_v1_dot_revocation__pb2.FetchRevocationListRequest.SerializeToString,
            capiscio_dot_v1_dot_revocation__pb2.FetchRevocationListResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ClearCache(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/capiscio.v1.RevocationService/ClearCache',
            capiscio_dot_v1_dot_revocation__pb2.ClearCacheRequest.SerializeToString,
            capiscio_dot_v1_dot_revocation__pb2.ClearCacheResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetCacheStats(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/capiscio.v1.RevocationService/GetCacheStats',
            capiscio_dot_v1_dot_revocation__pb2.GetCacheStatsRequest.SerializeToString,
            capiscio_dot_v1_dot_revocation__pb2.GetCacheStatsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



"""
gRPC client for capiscio-core MCPService.

Supports two connection modes:
- Embedded: Auto-downloads binary, starts locally, connects localhost
- External: Connects to user-provided CAPISCIO_CORE_ADDR

Usage:
    # Singleton pattern (recommended)
    client = await CoreClient.get_instance()
    response = await client.stub.EvaluateToolAccess(request)
    
    # Manual lifecycle
    client = CoreClient()
    await client.connect()
    # ... use client ...
    await client.close()
"""

from __future__ import annotations

import asyncio
import logging
import os
import socket
from pathlib import Path
from typing import Optional, TYPE_CHECKING

import grpc

from capiscio_mcp._core.version import MCP_VERSION
from capiscio_mcp._core.lifecycle import (
    ensure_binary,
    ProcessSupervisor,
)
from capiscio_mcp._core.health import wait_healthy, check_version_compatibility
from capiscio_mcp.errors import CoreConnectionError

if TYPE_CHECKING:
    from capiscio_mcp._proto.capiscio.v1 import mcp_pb2_grpc

logger = logging.getLogger(__name__)


def _get_running_loop_id() -> int:
    """Get the ID of the currently running event loop, or 0 if none."""
    try:
        loop = asyncio.get_running_loop()
        return id(loop)
    except RuntimeError:
        return 0


class CoreClient:
    """
    Async gRPC client to capiscio-core MCPService.
    
    Supports two modes:
    - Embedded: Auto-downloads binary, starts locally, connects localhost
    - External: Connects to user-provided CAPISCIO_CORE_ADDR
    
    The singleton pattern is event-loop aware: each event loop gets its own
    client instance. This ensures gRPC channels aren't shared across loops.
    
    Attributes:
        stub: The MCPService gRPC stub for making RPC calls
    """
    
    # Per-loop instances: {loop_id: CoreClient}
    _instances: dict[int, "CoreClient"] = {}
    _lock: Optional[asyncio.Lock] = None
    
    def __init__(self) -> None:
        """Initialize the client (not connected yet)."""
        self._channel: Optional[grpc.aio.Channel] = None
        self._stub: Optional["mcp_pb2_grpc.MCPServiceStub"] = None
        self._supervisor: Optional[ProcessSupervisor] = None
        self._port: Optional[int] = None
        self._connected = False
        self._loop_id: int = 0
    
    @classmethod
    def _get_lock(cls) -> asyncio.Lock:
        """Get or create the lock for the current event loop."""
        # Create lock lazily in the current event loop
        if cls._lock is None:
            cls._lock = asyncio.Lock()
        return cls._lock
    
    @classmethod
    async def get_instance(cls) -> "CoreClient":
        """
        Get or create the singleton client instance for the current event loop.
        
        Each event loop gets its own client instance. This prevents issues
        with gRPC channels being used across different event loops.
        
        Returns:
            Connected CoreClient instance
        """
        loop_id = _get_running_loop_id()
        
        async with cls._get_lock():
            # Check if we have a valid instance for this loop
            instance = cls._instances.get(loop_id)
            
            if instance is None or not instance._connected:
                # Create new instance for this loop
                instance = CoreClient()
                instance._loop_id = loop_id
                await instance.connect()
                cls._instances[loop_id] = instance
            
            return instance
    
    @classmethod
    async def reset(cls) -> None:
        """
        Reset the client instance for the current event loop.
        
        Call this to clean up resources. Safe to call multiple times.
        """
        loop_id = _get_running_loop_id()
        
        try:
            async with cls._get_lock():
                instance = cls._instances.pop(loop_id, None)
                if instance is not None:
                    await instance.close()
        except RuntimeError:
            # Lock may be invalid if loop is closing
            pass
    
    @classmethod
    async def reset_all(cls) -> None:
        """
        Reset all client instances (for testing/cleanup).
        """
        for loop_id, instance in list(cls._instances.items()):
            try:
                await instance.close()
            except Exception:
                pass
        cls._instances.clear()
        cls._lock = None
    
    # Backward compatibility alias
    @classmethod
    async def reset_instance(cls) -> None:
        """
        Reset the singleton instance (for testing).
        
        DEPRECATED: Use reset() instead.
        """
        await cls.reset()
    
    # Backward compatibility property for tests that directly set _instance
    @property
    def _instance(self) -> Optional["CoreClient"]:
        """Backward compatibility: get current loop's instance."""
        loop_id = _get_running_loop_id()
        return self._instances.get(loop_id)
    
    @_instance.setter
    def _instance(self, value: Optional["CoreClient"]) -> None:
        """Backward compatibility: set/clear current loop's instance."""
        loop_id = _get_running_loop_id() or id(asyncio.new_event_loop())
        if value is None:
            self._instances.pop(loop_id, None)
        else:
            self._instances[loop_id] = value
    
    async def connect(self, timeout: float = 30.0) -> None:
        """
        Connect to capiscio-core (embedded or external mode).
        
        Mode is determined by CAPISCIO_CORE_ADDR environment variable:
        - If set: External mode, connect to specified address
        - If not set: Embedded mode, download binary and start locally
        
        Args:
            timeout: Connection timeout in seconds
            
        Raises:
            CoreConnectionError: If connection fails
        """
        if self._connected:
            return
        
        addr = os.environ.get("CAPISCIO_CORE_ADDR")
        
        if addr:
            # External mode: user manages core process
            logger.info(f"Connecting to external capiscio-core at {addr}")
            self._channel = grpc.aio.insecure_channel(addr)
        else:
            # Embedded mode: download binary and start locally
            logger.info("Starting embedded capiscio-core")
            binary = await ensure_binary()
            self._port = await self._find_free_port()
            
            self._supervisor = ProcessSupervisor(binary, self._port)
            await self._supervisor.start()
            
            self._channel = grpc.aio.insecure_channel(f"localhost:{self._port}")
        
        # Import and create stub
        from capiscio_mcp._proto.capiscio.v1 import mcp_pb2_grpc
        self._stub = mcp_pb2_grpc.MCPServiceStub(self._channel)
        
        # Wait for healthy and check version
        try:
            await wait_healthy(self._stub, timeout=timeout)
            await check_version_compatibility(self._stub, MCP_VERSION)
        except Exception as e:
            await self.close()
            raise CoreConnectionError(f"Failed to connect to capiscio-core: {e}") from e
        
        self._connected = True
        logger.info("Successfully connected to capiscio-core")
    
    async def close(self) -> None:
        """Close connection and terminate embedded core if running."""
        self._connected = False
        
        if self._channel:
            await self._channel.close()
            self._channel = None
        
        if self._supervisor:
            await self._supervisor.stop()
            self._supervisor = None
        
        self._stub = None
        logger.debug("CoreClient closed")
    
    async def _find_free_port(self) -> int:
        """Find an available port for embedded core."""
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            s.bind(("", 0))
            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            return s.getsockname()[1]
    
    @property
    def stub(self) -> "mcp_pb2_grpc.MCPServiceStub":
        """
        Get the gRPC stub for MCPService.
        
        Raises:
            CoreConnectionError: If not connected
        """
        if self._stub is None:
            raise CoreConnectionError(
                "CoreClient not connected. Call connect() first or use get_instance()."
            )
        return self._stub
    
    @property
    def is_connected(self) -> bool:
        """Check if client is connected."""
        return self._connected
    
    async def __aenter__(self) -> "CoreClient":
        """Async context manager entry."""
        await self.connect()
        return self
    
    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        """Async context manager exit."""
        await self.close()



"""
MCP SDK Integrations for capiscio-mcp.
"""

from capiscio_mcp.integrations.mcp import (
    CapiscioMCPServer,
    CapiscioMCPClient,
    MCP_AVAILABLE,
)

__all__ = [
    "CapiscioMCPServer",
    "CapiscioMCPClient",
    "MCP_AVAILABLE",
]



"""
Tests for capiscio_mcp.pop module.

Tests PoP (Proof of Possession) primitives for RFC-007 MCP server identity.
"""

import time
import pytest

# Skip all tests if cryptography is not available
pytest.importorskip("cryptography")

from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PrivateKey

from capiscio_mcp.pop import (
    PoPRequest,
    PoPResponse,
    generate_nonce,
    generate_pop_request,
    sign_nonce,
    create_pop_response,
    verify_pop_signature,
    verify_pop_response,
    extract_public_key_from_did_key,
    public_key_to_did_key,
    load_private_key_from_pem,
    load_public_key_from_pem,
    PoPError,
    PoPNonceError,
    PoPSignatureError,
    PoPExpiredError,
    _base58_encode,
    _base58_decode,
)


class TestNonceGeneration:
    """Tests for nonce generation."""
    
    def test_generate_nonce_default_size(self):
        """Generate nonce with default size."""
        nonce = generate_nonce()
        # 32 bytes = 256 bits, base64url encoded ~ 43 chars without padding
        assert len(nonce) >= 40
        assert len(nonce) <= 50
        # Should be base64url (no padding)
        assert "=" not in nonce
    
    def test_generate_nonce_custom_size(self):
        """Generate nonce with custom size."""
        nonce = generate_nonce(16)  # 128 bits
        assert len(nonce) >= 20
        assert len(nonce) <= 25
    
    def test_generate_nonce_uniqueness(self):
        """Each nonce should be unique."""
        nonces = [generate_nonce() for _ in range(100)]
        assert len(set(nonces)) == 100


class TestPoPRequest:
    """Tests for PoPRequest dataclass."""
    
    def test_generate_pop_request(self):
        """Generate a PoP request."""
        req = generate_pop_request()
        assert req.client_nonce
        assert req.created_at > 0
        assert req.created_at <= time.time()
    
    def test_pop_request_to_meta(self):
        """Convert PoP request to _meta format."""
        req = PoPRequest(
            client_nonce="test-nonce-abc123",
            created_at=1700000000.0,
        )
        meta = req.to_meta()
        assert meta["capiscio_pop_nonce"] == "test-nonce-abc123"
        assert meta["capiscio_pop_created_at"] == 1700000000
    
    def test_pop_request_from_meta(self):
        """Parse PoP request from _meta."""
        meta = {
            "capiscio_pop_nonce": "test-nonce-xyz789",
            "capiscio_pop_created_at": 1700000000,
        }
        req = PoPRequest.from_meta(meta)
        assert req is not None
        assert req.client_nonce == "test-nonce-xyz789"
        assert req.created_at == 1700000000.0
    
    def test_pop_request_from_meta_missing_nonce(self):
        """Return None if nonce is missing."""
        assert PoPRequest.from_meta({}) is None
        assert PoPRequest.from_meta({"other": "field"}) is None
    
    def test_pop_request_from_meta_none(self):
        """Return None for None input."""
        assert PoPRequest.from_meta(None) is None
    
    def test_pop_request_is_expired(self):
        """Check expiry detection."""
        old_req = PoPRequest(
            client_nonce="old",
            created_at=time.time() - 400,  # 400 seconds ago
        )
        assert old_req.is_expired(max_age=300)  # 5 minute limit
        
        new_req = PoPRequest(
            client_nonce="new",
            created_at=time.time(),
        )
        assert not new_req.is_expired(max_age=300)


class TestPoPResponse:
    """Tests for PoPResponse dataclass."""
    
    def test_pop_response_to_meta(self):
        """Convert PoP response to _meta format."""
        resp = PoPResponse(
            nonce_signature="eyJ...signature...",
            signed_at=1700000100.0,
        )
        meta = resp.to_meta()
        assert meta["capiscio_pop_signature"] == "eyJ...signature..."
        assert meta["capiscio_pop_signed_at"] == 1700000100
    
    def test_pop_response_from_meta(self):
        """Parse PoP response from _meta."""
        meta = {
            "capiscio_pop_signature": "eyJ...sig...",
            "capiscio_pop_signed_at": 1700000100,
        }
        resp = PoPResponse.from_meta(meta)
        assert resp is not None
        assert resp.nonce_signature == "eyJ...sig..."
        assert resp.signed_at == 1700000100.0
    
    def test_pop_response_from_meta_missing(self):
        """Return None if signature is missing."""
        assert PoPResponse.from_meta({}) is None
        assert PoPResponse.from_meta({"other": "field"}) is None


class TestPoPSigning:
    """Tests for PoP signing functions."""
    
    @pytest.fixture
    def key_pair(self):
        """Generate a test Ed25519 key pair."""
        private_key = Ed25519PrivateKey.generate()
        public_key = private_key.public_key()
        return private_key, public_key
    
    def test_sign_nonce(self, key_pair):
        """Sign a nonce and get JWS."""
        private_key, _ = key_pair
        nonce = "test-nonce-123"
        key_id = "did:key:z6Mk...#keys-1"
        
        jws = sign_nonce(nonce, private_key, key_id)
        
        # JWS compact format: header.payload.signature
        parts = jws.split(".")
        assert len(parts) == 3
        # Each part should be base64url encoded
        assert all(p for p in parts)
    
    def test_create_pop_response(self, key_pair):
        """Create a full PoP response."""
        private_key, _ = key_pair
        
        request = PoPRequest(
            client_nonce="challenge-nonce",
            created_at=time.time(),
        )
        
        response = create_pop_response(
            request=request,
            private_key=private_key,
            key_id="did:key:z6Mk...#keys-1",
        )
        
        assert response.nonce_signature
        assert response.signed_at > 0


class TestPoPVerification:
    """Tests for PoP verification functions."""
    
    @pytest.fixture
    def key_pair(self):
        """Generate a test Ed25519 key pair."""
        private_key = Ed25519PrivateKey.generate()
        public_key = private_key.public_key()
        return private_key, public_key
    
    def test_verify_pop_signature(self, key_pair):
        """Verify a valid PoP signature."""
        private_key, public_key = key_pair
        nonce = "verify-test-nonce"
        
        jws = sign_nonce(nonce, private_key, "test-key-id")
        
        # Should not raise
        verify_pop_signature(jws, nonce, public_key)
    
    def test_verify_pop_signature_wrong_key(self, key_pair):
        """Verification fails with wrong public key."""
        private_key, _ = key_pair
        wrong_private = Ed25519PrivateKey.generate()
        wrong_public = wrong_private.public_key()
        
        nonce = "nonce-123"
        jws = sign_nonce(nonce, private_key, "key-id")
        
        with pytest.raises(PoPSignatureError):
            verify_pop_signature(jws, nonce, wrong_public)
    
    def test_verify_pop_signature_wrong_nonce(self, key_pair):
        """Verification fails with wrong expected nonce."""
        private_key, public_key = key_pair
        
        jws = sign_nonce("original-nonce", private_key, "key-id")
        
        with pytest.raises(PoPSignatureError, match="mismatch"):
            verify_pop_signature(jws, "different-nonce", public_key)
    
    def test_verify_pop_response_full_flow(self, key_pair):
        """Full PoP request/response verification flow."""
        private_key, public_key = key_pair
        
        # Client generates request
        request = generate_pop_request()
        
        # Server signs response
        response = create_pop_response(
            request=request,
            private_key=private_key,
            key_id="test-server#keys-1",
        )
        
        # Client verifies response
        verify_pop_response(request, response, public_key)
    
    def test_verify_pop_response_expired(self, key_pair):
        """Verification fails if request expired."""
        private_key, public_key = key_pair
        
        # Old request
        request = PoPRequest(
            client_nonce="old-nonce",
            created_at=time.time() - 600,  # 10 minutes ago
        )
        
        response = create_pop_response(request, private_key, "key-id")
        
        with pytest.raises(PoPExpiredError):
            verify_pop_response(request, response, public_key, max_age=300)


class TestDIDKeyConversion:
    """Tests for DID:key conversion utilities."""
    
    @pytest.fixture
    def key_pair(self):
        """Generate a test Ed25519 key pair."""
        private_key = Ed25519PrivateKey.generate()
        public_key = private_key.public_key()
        return private_key, public_key
    
    def test_public_key_to_did_key(self, key_pair):
        """Convert public key to did:key."""
        _, public_key = key_pair
        did = public_key_to_did_key(public_key)
        
        assert did.startswith("did:key:z6Mk")
    
    def test_extract_public_key_from_did_key(self, key_pair):
        """Extract public key from did:key."""
        _, public_key = key_pair
        did = public_key_to_did_key(public_key)
        
        extracted = extract_public_key_from_did_key(did)
        
        # Should be able to use for verification
        from cryptography.hazmat.primitives import serialization
        
        original_bytes = public_key.public_bytes(
            encoding=serialization.Encoding.Raw,
            format=serialization.PublicFormat.Raw,
        )
        extracted_bytes = extracted.public_bytes(
            encoding=serialization.Encoding.Raw,
            format=serialization.PublicFormat.Raw,
        )
        assert original_bytes == extracted_bytes
    
    def test_did_key_roundtrip(self, key_pair):
        """Roundtrip conversion preserves key."""
        _, public_key = key_pair
        
        did = public_key_to_did_key(public_key)
        extracted = extract_public_key_from_did_key(did)
        did2 = public_key_to_did_key(extracted)
        
        assert did == did2
    
    def test_extract_invalid_did_key(self):
        """Raise on invalid did:key format."""
        with pytest.raises(ValueError):
            extract_public_key_from_did_key("did:web:example.com")
        
        with pytest.raises(ValueError):
            extract_public_key_from_did_key("not-a-did")


class TestBase58:
    """Tests for base58btc encoding."""
    
    def test_base58_roundtrip(self):
        """Roundtrip encoding preserves data."""
        import os
        data = os.urandom(32)
        encoded = _base58_encode(data)
        decoded = _base58_decode(encoded)
        assert data == decoded
    
    def test_base58_leading_zeros(self):
        """Handle leading zeros correctly."""
        data = b"\x00\x00\x01\x02\x03"
        encoded = _base58_encode(data)
        assert encoded.startswith("11")  # Leading zeros become '1's
        decoded = _base58_decode(encoded)
        assert data == decoded
    
    def test_base58_empty(self):
        """Handle empty input."""
        assert _base58_encode(b"") == ""
        assert _base58_decode("") == b""


class TestMetaRoundtrip:
    """Tests for _meta field roundtrip."""
    
    @pytest.fixture
    def key_pair(self):
        """Generate a test Ed25519 key pair."""
        private_key = Ed25519PrivateKey.generate()
        public_key = private_key.public_key()
        return private_key, public_key
    
    def test_full_meta_roundtrip(self, key_pair):
        """Full _meta roundtrip between client and server."""
        private_key, public_key = key_pair
        
        # Client creates request and puts in _meta
        client_request = generate_pop_request()
        request_meta = client_request.to_meta()
        
        # Server parses request from _meta
        server_request = PoPRequest.from_meta(request_meta)
        assert server_request is not None
        assert server_request.client_nonce == client_request.client_nonce
        
        # Server creates response
        server_response = create_pop_response(server_request, private_key, "key-id")
        response_meta = {
            "capiscio_server_did": "did:key:z6MkTest...",
            **server_response.to_meta(),
        }
        
        # Client parses response from _meta
        client_response = PoPResponse.from_meta(response_meta)
        assert client_response is not None
        
        # Client verifies PoP
        verify_pop_response(client_request, client_response, public_key)


class TestPEMLoading:
    """Tests for PEM key loading."""
    
    @pytest.fixture
    def key_pair_pem(self):
        """Generate key pair and PEM representation."""
        from cryptography.hazmat.primitives import serialization
        
        private_key = Ed25519PrivateKey.generate()
        public_key = private_key.public_key()
        
        private_pem = private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption(),
        )
        
        public_pem = public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo,
        )
        
        return private_pem, public_pem
    
    def test_load_private_key_from_pem(self, key_pair_pem):
        """Load private key from PEM."""
        private_pem, _ = key_pair_pem
        key = load_private_key_from_pem(private_pem)
        assert key is not None
    
    def test_load_public_key_from_pem(self, key_pair_pem):
        """Load public key from PEM."""
        _, public_pem = key_pair_pem
        key = load_public_key_from_pem(public_pem)
        assert key is not None
    
    def test_load_key_from_pem_string(self, key_pair_pem):
        """Load key from PEM string (not bytes)."""
        private_pem, _ = key_pair_pem
        key = load_private_key_from_pem(private_pem.decode("utf-8"))
        assert key is not None



"""Tests for capiscio_mcp._core.client module."""

import asyncio
import os
from unittest.mock import AsyncMock, MagicMock, patch
import pytest

from capiscio_mcp._core.client import CoreClient


@pytest.fixture(autouse=True)
def reset_singleton():
    """Reset CoreClient singleton before and after each test."""
    # Reset class-level state for new per-loop-instance pattern
    CoreClient._instances.clear()
    CoreClient._lock = None
    yield
    CoreClient._instances.clear()
    CoreClient._lock = None


class TestCoreClientSingleton:
    """Tests for CoreClient singleton pattern."""
    
    @pytest.mark.asyncio
    async def test_get_instance_returns_same_instance(self):
        """get_instance should return same instance within same event loop."""
        with patch.object(CoreClient, "connect", new_callable=AsyncMock):
            # Mark as connected so connect isn't called again
            instance1 = await CoreClient.get_instance()
            instance1._connected = True
            instance2 = await CoreClient.get_instance()
            
            assert instance1 is instance2
    
    @pytest.mark.asyncio
    async def test_get_instance_calls_connect_once(self):
        """get_instance should only call connect once when already connected."""
        with patch.object(CoreClient, "connect", new_callable=AsyncMock) as mock_connect:
            # First call creates and connects
            instance = await CoreClient.get_instance()
            # Mark as connected to prevent reconnection
            instance._connected = True
            
            await CoreClient.get_instance()
            await CoreClient.get_instance()
            
            # Should only connect once since _connected is True
            mock_connect.assert_called_once()
    
    def test_reset_singleton(self):
        """Should be able to reset singleton for testing."""
        # Add a mock instance to _instances
        loop_id = 12345
        CoreClient._instances[loop_id] = MagicMock()
        assert len(CoreClient._instances) > 0
        
        # Clear it
        CoreClient._instances.clear()
        assert len(CoreClient._instances) == 0


class TestCoreClientEmbeddedMode:
    """Tests for embedded mode (auto-start binary)."""
    
    @pytest.mark.asyncio
    async def test_embedded_mode_when_no_addr(self):
        """Should use embedded mode when CAPISCIO_CORE_ADDR not set."""
        # Clear environment variable
        env_without_addr = {k: v for k, v in os.environ.items() if k != "CAPISCIO_CORE_ADDR"}
        
        with patch.dict(os.environ, env_without_addr, clear=True):
            # Mock the imports in client.py - need to patch where they're used
            with patch("capiscio_mcp._core.client.ensure_binary", new_callable=AsyncMock) as mock_ensure:
                mock_ensure.return_value = "/usr/local/bin/capiscio"
                
                with patch("capiscio_mcp._core.client.ProcessSupervisor") as mock_supervisor_cls:
                    mock_supervisor = MagicMock()
                    mock_supervisor.start = AsyncMock()
                    mock_supervisor.stop = AsyncMock()
                    mock_supervisor_cls.return_value = mock_supervisor
                    
                    with patch("capiscio_mcp._core.client.grpc.aio.insecure_channel") as mock_channel:
                        mock_channel_instance = MagicMock()
                        mock_channel_instance.close = AsyncMock()
                        mock_channel.return_value = mock_channel_instance
                        
                        with patch("capiscio_mcp._core.client.wait_healthy", new_callable=AsyncMock):
                            with patch("capiscio_mcp._core.client.check_version_compatibility", new_callable=AsyncMock):
                                with patch("capiscio_mcp._proto.capiscio.v1.mcp_pb2_grpc.MCPServiceStub"):
                                    client = CoreClient()
                                    await client.connect()
                                    
                                    # Should have downloaded/ensured binary
                                    mock_ensure.assert_called_once()
    
    @pytest.mark.asyncio
    async def test_embedded_mode_finds_free_port(self):
        """Embedded mode should find an available port."""
        client = CoreClient()
        port = await client._find_free_port()
        
        assert isinstance(port, int)
        assert port > 0
        assert port < 65536


class TestCoreClientExternalMode:
    """Tests for external mode (user-managed core)."""
    
    @pytest.mark.asyncio
    async def test_external_mode_when_addr_set(self):
        """Should use external mode when CAPISCIO_CORE_ADDR is set."""
        with patch.dict(os.environ, {"CAPISCIO_CORE_ADDR": "localhost:50051"}):
            with patch("capiscio_mcp._core.client.grpc.aio.insecure_channel") as mock_channel:
                mock_channel_instance = MagicMock()
                mock_channel_instance.close = AsyncMock()
                mock_channel.return_value = mock_channel_instance
                
                with patch("capiscio_mcp._core.client.wait_healthy", new_callable=AsyncMock):
                    with patch("capiscio_mcp._core.client.check_version_compatibility", new_callable=AsyncMock):
                        with patch("capiscio_mcp._proto.capiscio.v1.mcp_pb2_grpc.MCPServiceStub"):
                            client = CoreClient()
                            await client.connect()
                            
                            # Should connect to specified address
                            mock_channel.assert_called_once_with("localhost:50051")
    
    @pytest.mark.asyncio
    async def test_external_mode_custom_port(self):
        """External mode should use custom port from env."""
        with patch.dict(os.environ, {"CAPISCIO_CORE_ADDR": "core.example.com:9999"}):
            with patch("capiscio_mcp._core.client.grpc.aio.insecure_channel") as mock_channel:
                mock_channel_instance = MagicMock()
                mock_channel_instance.close = AsyncMock()
                mock_channel.return_value = mock_channel_instance
                
                with patch("capiscio_mcp._core.client.wait_healthy", new_callable=AsyncMock):
                    with patch("capiscio_mcp._core.client.check_version_compatibility", new_callable=AsyncMock):
                        with patch("capiscio_mcp._proto.capiscio.v1.mcp_pb2_grpc.MCPServiceStub"):
                            client = CoreClient()
                            await client.connect()
                            
                            mock_channel.assert_called_once_with("core.example.com:9999")


class TestCoreClientVersionHandshake:
    """Tests for version compatibility handshake."""
    
    @pytest.mark.asyncio
    async def test_incompatible_version_raises(self):
        """Should raise error when core version is incompatible."""
        with patch.dict(os.environ, {"CAPISCIO_CORE_ADDR": "localhost:50051"}):
            with patch("capiscio_mcp._core.client.grpc.aio.insecure_channel") as mock_channel:
                mock_channel_instance = MagicMock()
                mock_channel_instance.close = AsyncMock()
                mock_channel.return_value = mock_channel_instance
                
                with patch("capiscio_mcp._core.client.wait_healthy", new_callable=AsyncMock):
                    with patch("capiscio_mcp._core.client.check_version_compatibility", new_callable=AsyncMock) as mock_compat:
                        mock_compat.side_effect = RuntimeError("version mismatch")
                        
                        with patch("capiscio_mcp._proto.capiscio.v1.mcp_pb2_grpc.MCPServiceStub"):
                            client = CoreClient()
                            
                            from capiscio_mcp.errors import CoreConnectionError
                            with pytest.raises(CoreConnectionError) as exc_info:
                                await client.connect()
                            
                            assert "version" in str(exc_info.value).lower()
    
    @pytest.mark.asyncio
    async def test_compatible_version_continues(self):
        """Should continue when core version is compatible."""
        with patch.dict(os.environ, {"CAPISCIO_CORE_ADDR": "localhost:50051"}):
            with patch("capiscio_mcp._core.client.grpc.aio.insecure_channel") as mock_channel:
                mock_channel_instance = MagicMock()
                mock_channel_instance.close = AsyncMock()
                mock_channel.return_value = mock_channel_instance
                
                with patch("capiscio_mcp._core.client.wait_healthy", new_callable=AsyncMock):
                    with patch("capiscio_mcp._core.client.check_version_compatibility", new_callable=AsyncMock):
                        with patch("capiscio_mcp._proto.capiscio.v1.mcp_pb2_grpc.MCPServiceStub"):
                            client = CoreClient()
                            await client.connect()
                            
                            # Should not raise
                            assert client._connected


class TestCoreClientHealthCheck:
    """Tests for health check on connect."""
    
    @pytest.mark.asyncio
    async def test_waits_for_healthy(self):
        """Should wait for core to be healthy."""
        with patch.dict(os.environ, {"CAPISCIO_CORE_ADDR": "localhost:50051"}):
            with patch("capiscio_mcp._core.client.grpc.aio.insecure_channel") as mock_channel:
                mock_channel_instance = MagicMock()
                mock_channel_instance.close = AsyncMock()
                mock_channel.return_value = mock_channel_instance
                
                with patch("capiscio_mcp._core.client.wait_healthy", new_callable=AsyncMock) as mock_health:
                    with patch("capiscio_mcp._core.client.check_version_compatibility", new_callable=AsyncMock):
                        with patch("capiscio_mcp._proto.capiscio.v1.mcp_pb2_grpc.MCPServiceStub"):
                            client = CoreClient()
                            await client.connect()
                            
                            mock_health.assert_called_once()
    
    @pytest.mark.asyncio
    async def test_health_timeout_raises(self):
        """Should raise error if health check times out."""
        with patch.dict(os.environ, {"CAPISCIO_CORE_ADDR": "localhost:50051"}):
            with patch("capiscio_mcp._core.client.grpc.aio.insecure_channel") as mock_channel:
                mock_channel_instance = MagicMock()
                mock_channel_instance.close = AsyncMock()
                mock_channel.return_value = mock_channel_instance
                
                with patch("capiscio_mcp._core.client.wait_healthy", new_callable=AsyncMock) as mock_health:
                    mock_health.side_effect = asyncio.TimeoutError()
                    
                    with patch("capiscio_mcp._proto.capiscio.v1.mcp_pb2_grpc.MCPServiceStub"):
                        client = CoreClient()
                        
                        from capiscio_mcp.errors import CoreConnectionError
                        with pytest.raises(CoreConnectionError):
                            await client.connect()


class TestCoreClientClose:
    """Tests for closing connection."""
    
    @pytest.mark.asyncio
    async def test_close_channel(self):
        """Should close gRPC channel."""
        client = CoreClient()
        
        mock_channel = MagicMock()
        mock_channel.close = AsyncMock()
        client._channel = mock_channel
        
        await client.close()
        
        mock_channel.close.assert_called_once()
    
    @pytest.mark.asyncio
    async def test_close_stops_supervisor(self):
        """Should stop supervisor on close."""
        client = CoreClient()
        
        mock_channel = MagicMock()
        mock_channel.close = AsyncMock()
        client._channel = mock_channel
        
        mock_supervisor = MagicMock()
        mock_supervisor.stop = AsyncMock()
        client._supervisor = mock_supervisor
        
        await client.close()
        
        mock_supervisor.stop.assert_called_once()
    
    @pytest.mark.asyncio
    async def test_close_without_supervisor(self):
        """Should handle close when no supervisor."""
        client = CoreClient()
        
        mock_channel = MagicMock()
        mock_channel.close = AsyncMock()
        client._channel = mock_channel
        client._supervisor = None
        
        # Should not raise
        await client.close()


class TestCoreClientStub:
    """Tests for gRPC stub property."""
    
    def test_stub_property(self):
        """stub property should return the gRPC stub."""
        client = CoreClient()
        
        mock_stub = MagicMock()
        client._stub = mock_stub
        
        assert client.stub is mock_stub
    
    def test_stub_raises_when_not_connected(self):
        """stub should raise when not connected."""
        client = CoreClient()
        
        from capiscio_mcp.errors import CoreConnectionError
        with pytest.raises(CoreConnectionError):
            _ = client.stub


class TestCoreClientContextManager:
    """Tests for async context manager."""
    
    @pytest.mark.asyncio
    async def test_context_manager_connects_and_closes(self):
        """Context manager should connect on enter and close on exit."""
        with patch.dict(os.environ, {"CAPISCIO_CORE_ADDR": "localhost:50051"}):
            with patch("capiscio_mcp._core.client.grpc.aio.insecure_channel") as mock_channel:
                mock_channel_instance = MagicMock()
                mock_channel_instance.close = AsyncMock()
                mock_channel.return_value = mock_channel_instance
                
                with patch("capiscio_mcp._core.client.wait_healthy", new_callable=AsyncMock):
                    with patch("capiscio_mcp._core.client.check_version_compatibility", new_callable=AsyncMock):
                        with patch("capiscio_mcp._proto.capiscio.v1.mcp_pb2_grpc.MCPServiceStub"):
                            async with CoreClient() as client:
                                assert client._connected
                            
                            # After exit, should be closed
                            assert not client._connected


class TestCoreClientIsConnected:
    """Tests for is_connected property."""
    
    def test_is_connected_false_initially(self):
        """is_connected should be False initially."""
        client = CoreClient()
        assert not client.is_connected
    
    @pytest.mark.asyncio
    async def test_is_connected_true_after_connect(self):
        """is_connected should be True after connect."""
        with patch.dict(os.environ, {"CAPISCIO_CORE_ADDR": "localhost:50051"}):
            with patch("capiscio_mcp._core.client.grpc.aio.insecure_channel") as mock_channel:
                mock_channel_instance = MagicMock()
                mock_channel_instance.close = AsyncMock()
                mock_channel.return_value = mock_channel_instance
                
                with patch("capiscio_mcp._core.client.wait_healthy", new_callable=AsyncMock):
                    with patch("capiscio_mcp._core.client.check_version_compatibility", new_callable=AsyncMock):
                        with patch("capiscio_mcp._proto.capiscio.v1.mcp_pb2_grpc.MCPServiceStub"):
                            client = CoreClient()
                            await client.connect()
                            
                            assert client.is_connected



"""Tests for capiscio_mcp.integrations.mcp module."""

import pytest
from unittest.mock import AsyncMock, MagicMock, patch

# Test imports work
from capiscio_mcp.integrations.mcp import (
    CapiscioMCPServer,
    CapiscioMCPClient,
    MCP_AVAILABLE,
    MCP_CLIENT_AVAILABLE,
)
from capiscio_mcp.types import ServerState
from capiscio_mcp.errors import GuardError, ServerVerifyError

# Skip tests that require MCP package if not installed
requires_mcp = pytest.mark.skipif(not MCP_AVAILABLE, reason="MCP package not installed")
requires_mcp_client = pytest.mark.skipif(not MCP_CLIENT_AVAILABLE, reason="MCP client not installed")


class TestMCPAvailability:
    """Tests for MCP SDK availability detection."""
    
    def test_mcp_available_flag_exists(self):
        """MCP_AVAILABLE flag should exist."""
        assert isinstance(MCP_AVAILABLE, bool)
    
    @patch.dict("sys.modules", {"mcp": None, "mcp.server": None, "mcp.client": None})
    def test_mcp_not_available_without_package(self):
        """MCP should not be available without mcp package."""
        # Re-import to pick up mocked modules
        # Note: actual behavior depends on import order
        pass


@requires_mcp
class TestCapiscioMCPServer:
    """Tests for CapiscioMCPServer class."""
    
    def test_init_with_required_params(self):
        """Should initialize with required parameters."""
        with patch("capiscio_mcp.integrations.mcp.MCP_AVAILABLE", True):
            with patch("capiscio_mcp.integrations.mcp.FastMCP"):
                server = CapiscioMCPServer(
                    name="filesystem",
                    did="did:web:mcp.example.com:servers:fs",
                )
                
                assert server.name == "filesystem"
                assert server.did == "did:web:mcp.example.com:servers:fs"
    
    def test_init_with_badge(self):
        """Should accept optional badge."""
        with patch("capiscio_mcp.integrations.mcp.MCP_AVAILABLE", True):
            with patch("capiscio_mcp.integrations.mcp.FastMCP"):
                server = CapiscioMCPServer(
                    name="filesystem",
                    did="did:web:mcp.example.com:servers:fs",
                    badge="eyJhbGc...",
                )
                
                assert server.badge == "eyJhbGc..."
    
    def test_init_with_default_trust_level(self):
        """Should accept default_min_trust_level."""
        with patch("capiscio_mcp.integrations.mcp.MCP_AVAILABLE", True):
            with patch("capiscio_mcp.integrations.mcp.FastMCP"):
                server = CapiscioMCPServer(
                    name="filesystem",
                    did="did:web:mcp.example.com:servers:fs",
                    default_min_trust_level=2,
                )
                
                assert server.default_min_trust_level == 2
    
    def test_init_raises_without_mcp(self):
        """Should raise ImportError without MCP SDK."""
        with patch("capiscio_mcp.integrations.mcp.MCP_AVAILABLE", False):
            with pytest.raises(ImportError) as exc_info:
                server = CapiscioMCPServer(
                    name="test",
                    did="did:web:example.com",
                )
            
            assert "pip install capiscio-mcp[mcp]" in str(exc_info.value)
    
    def test_tool_decorator(self):
        """tool decorator should register guarded tools."""
        with patch("capiscio_mcp.integrations.mcp.MCP_AVAILABLE", True):
            with patch("capiscio_mcp.integrations.mcp.FastMCP") as mock_fastmcp:
                mock_instance = MagicMock()
                mock_fastmcp.return_value = mock_instance
                
                server = CapiscioMCPServer(
                    name="filesystem",
                    did="did:web:mcp.example.com:servers:fs",
                )
                
                @server.tool(min_trust_level=2)
                async def read_file(path: str) -> str:
                    return f"Contents of {path}"
                
                # Should have registered the tool internally
                assert "read_file" in server._tools
                assert server._tool_configs["read_file"].min_trust_level == 2
    
    def test_tool_decorator_with_custom_name(self):
        """tool decorator should accept custom tool name."""
        with patch("capiscio_mcp.integrations.mcp.MCP_AVAILABLE", True):
            with patch("capiscio_mcp.integrations.mcp.FastMCP") as mock_fastmcp:
                mock_instance = MagicMock()
                mock_fastmcp.return_value = mock_instance
                
                server = CapiscioMCPServer(
                    name="filesystem",
                    did="did:web:mcp.example.com:servers:fs",
                )
                
                @server.tool(name="fs.read", min_trust_level=1)
                async def read_file(path: str) -> str:
                    return f"Contents of {path}"
                
                # Should have registered with custom name
                assert "fs.read" in server._tools
                assert "read_file" not in server._tools
    
    def test_tool_uses_default_trust_level(self):
        """tool should use server's default_min_trust_level."""
        with patch("capiscio_mcp.integrations.mcp.MCP_AVAILABLE", True):
            with patch("capiscio_mcp.integrations.mcp.FastMCP"):
                server = CapiscioMCPServer(
                    name="filesystem",
                    did="did:web:mcp.example.com:servers:fs",
                    default_min_trust_level=3,
                )
                
                @server.tool()  # No min_trust_level specified
                async def test_tool() -> str:
                    return "result"
                
                # Tool should use default trust level 3
    
    def test_server_property_returns_underlying_server(self):
        """server property should return underlying FastMCP server."""
        with patch("capiscio_mcp.integrations.mcp.MCP_AVAILABLE", True):
            with patch("capiscio_mcp.integrations.mcp.FastMCP") as mock_fastmcp:
                mock_instance = MagicMock()
                mock_fastmcp.return_value = mock_instance
                
                server = CapiscioMCPServer(
                    name="filesystem",
                    did="did:web:mcp.example.com:servers:fs",
                )
                
                assert server.server is mock_instance


@requires_mcp_client
class TestCapiscioMCPClient:
    """Tests for CapiscioMCPClient class."""
    
    def test_init_with_server_url(self):
        """Should initialize with server_url parameter."""
        with patch("capiscio_mcp.integrations.mcp.MCP_CLIENT_AVAILABLE", True):
            client = CapiscioMCPClient(
                server_url="https://mcp.example.com",
            )
            
            assert client.server_url == "https://mcp.example.com"
    
    def test_init_with_command(self):
        """Should initialize with command parameter for stdio transport."""
        with patch("capiscio_mcp.integrations.mcp.MCP_CLIENT_AVAILABLE", True):
            client = CapiscioMCPClient(
                command="python",
                args=["server.py"],
            )
            
            assert client.command == "python"
            assert client.args == ["server.py"]
    
    def test_init_raises_without_transport(self):
        """Should raise ValueError if neither server_url nor command provided."""
        with patch("capiscio_mcp.integrations.mcp.MCP_CLIENT_AVAILABLE", True):
            with pytest.raises(ValueError) as exc_info:
                client = CapiscioMCPClient()
            
            assert "server_url or command" in str(exc_info.value)
    
    def test_init_with_min_trust_level(self):
        """Should accept min_trust_level."""
        with patch("capiscio_mcp.integrations.mcp.MCP_CLIENT_AVAILABLE", True):
            client = CapiscioMCPClient(
                command="python",
                args=["server.py"],
                min_trust_level=2,
            )
            
            assert client.min_trust_level == 2
    
    def test_init_with_fail_on_unverified(self):
        """Should accept fail_on_unverified flag."""
        with patch("capiscio_mcp.integrations.mcp.MCP_CLIENT_AVAILABLE", True):
            client = CapiscioMCPClient(
                command="python",
                args=["server.py"],
                fail_on_unverified=False,
            )
            
            assert client.fail_on_unverified is False
    
    def test_init_raises_without_mcp(self):
        """Should raise ImportError without MCP SDK."""
        with patch("capiscio_mcp.integrations.mcp.MCP_CLIENT_AVAILABLE", False):
            with pytest.raises(ImportError) as exc_info:
                client = CapiscioMCPClient(
                    command="python",
                    args=["server.py"],
                )
            
            assert "pip install capiscio-mcp[mcp]" in str(exc_info.value)
    
    @pytest.mark.asyncio
    async def test_context_manager_cleanup(self):
        """Context manager should clean up on exit."""
        with patch("capiscio_mcp.integrations.mcp.MCP_CLIENT_AVAILABLE", True):
            with patch("capiscio_mcp.integrations.mcp.stdio_client") as mock_stdio:
                with patch("capiscio_mcp.integrations.mcp.McpClientSession") as mock_session:
                    # Set up mocks
                    mock_cm = AsyncMock()
                    mock_cm.__aenter__.return_value = (MagicMock(), MagicMock())
                    mock_stdio.return_value = mock_cm
                    
                    mock_session_instance = AsyncMock()
                    mock_session_instance.initialize = AsyncMock()
                    mock_session.return_value = mock_session_instance
                    
                    client = CapiscioMCPClient(
                        command="python",
                        args=["server.py"],
                    )
                    
                    async with client:
                        pass
                    
                    # After exit, session should be cleaned up
                    assert client._session is None
                    assert client._context_manager is None
    
    @pytest.mark.asyncio
    async def test_http_transport_not_implemented(self):
        """HTTP transport should raise NotImplementedError."""
        with patch("capiscio_mcp.integrations.mcp.MCP_CLIENT_AVAILABLE", True):
            client = CapiscioMCPClient(
                server_url="https://mcp.example.com",
            )
            
            with pytest.raises(NotImplementedError) as exc_info:
                async with client:
                    pass
            
            assert "HTTP transport" in str(exc_info.value)
    
    def test_server_state_property_before_connect(self):
        """server_state property should return None before connect."""
        with patch("capiscio_mcp.integrations.mcp.MCP_CLIENT_AVAILABLE", True):
            client = CapiscioMCPClient(
                command="python",
                args=["server.py"],
            )
            
            assert client.server_state is None
    
    def test_server_trust_level_property_before_connect(self):
        """server_trust_level property should return None before connect."""
        with patch("capiscio_mcp.integrations.mcp.MCP_CLIENT_AVAILABLE", True):
            client = CapiscioMCPClient(
                command="python",
                args=["server.py"],
            )
            
            assert client.server_trust_level is None
    
    @pytest.mark.asyncio
    async def test_call_tool_raises_when_not_connected(self):
        """call_tool should raise when not connected."""
        with patch("capiscio_mcp.integrations.mcp.MCP_CLIENT_AVAILABLE", True):
            client = CapiscioMCPClient(
                command="python",
                args=["server.py"],
            )
            
            with pytest.raises(RuntimeError) as exc_info:
                await client.call_tool("test", {})
            
            assert "not connected" in str(exc_info.value).lower()
    
    @pytest.mark.asyncio
    async def test_list_tools_raises_when_not_connected(self):
        """list_tools should raise when not connected."""
        with patch("capiscio_mcp.integrations.mcp.MCP_CLIENT_AVAILABLE", True):
            client = CapiscioMCPClient(
                command="python",
                args=["server.py"],
            )
            
            with pytest.raises(RuntimeError) as exc_info:
                await client.list_tools()
            
            assert "not connected" in str(exc_info.value).lower()
